import enum

from lucina.utils import ordered_enum


def test_ordered_enum():
    @ordered_enum
    class Enum(enum.Enum):
        A = 'a'
        B = 'b'
        C = 'c'

    assert Enum.A.rank == 0
    assert Enum.B.rank == 1
    assert Enum.C.rank == 2

    assert Enum('b').rank == 1



from lucina.cell import Cell
from lucina.cell import CellType
from lucina.cell import SlideType


def test_cell():
    cell1 = Cell()
    assert cell1.cell_type is CellType.MARKDOWN
    assert cell1.slide_type is SlideType.CONTINUE
    assert cell1.source == []

    cell2 = Cell(CellType.CODE, SlideType.SLIDE, ['foo', 'bar'])
    assert cell2.cell_type is CellType.CODE
    assert cell2.slide_type is SlideType.SLIDE
    assert cell2.source == ['foo', 'bar']

    cell3 = Cell('code', 'slide', ['foo', 'bar'])
    assert cell3.cell_type is CellType.CODE
    assert cell3.slide_type is SlideType.SLIDE
    assert cell3.source == ['foo', 'bar']

    assert cell1 != cell2
    assert cell2 == cell3



import enum
from dataclasses import dataclass
from dataclasses import field
from typing import List

from lucina.utils import ordered_enum


class CellType(enum.Enum):
    MARKDOWN = 'markdown'
    CODE = 'code'


@ordered_enum
class SlideType(enum.Enum):
    SLIDE = 'slide'
    SUBSLIDE = 'subslide'
    FRAGMENT = 'fragment'
    CONTINUE = '-'
    NOTES = 'notes'
    SKIP = 'skip'


@dataclass
class Cell:
    cell_type: CellType = CellType.MARKDOWN
    slide_type: SlideType = SlideType.CONTINUE
    source: List[str] = field(default_factory=list)

    def __post_init__(self):
        self.cell_type = CellType(self.cell_type)
        self.slide_type = SlideType(self.slide_type)



from minimd.tokenizer import Token

from lucina.cell import Cell
from lucina.cell import SlideType
from lucina.parser import SplitRules
from lucina.parser import clean_source
from lucina.parser import parse_cells


TOKENS = [
    Token.FILE(),
    Token.TITLE('# 111\n', level=1),
    Token.AFTER_TITLE(level=1),
    Token.LINE('\n'),
    Token.LINE('* bullet point\n'),
    Token.LINE('* bullet point 2\n'),
    Token.LINE('\n'),
    Token.START_CODE('```python\n', language='python', skip=False),
    Token.LINE('def random():\n'),
    Token.LINE('    return 4\n'),
    Token.END_CODE('```\n'),
    Token.LINE('\n'),
    Token.SPLIT('---\n'),
    Token.LINE('\n'),
    Token.START_CODE('```python\n', language='python', skip=False),
    Token.LINE('print(random())\n'),
    Token.END_CODE('```\n'),
    Token.LINE('\n'),
    Token.SPLIT('---\n'),
    Token.LINE('\n'),
    Token.LINE('Text.\n'),
    Token.AFTER_FILE(),
    Token.FILE(),
    Token.AFTER_FILE(),
    Token.FILE(),
    Token.TITLE('## 222\n', level=2),
    Token.AFTER_TITLE(level=2),
    Token.LINE('\n'),
    Token.START_CODE('```python skip\n', language='python', skip=True),
    Token.LINE('import itertools\n'),
    Token.END_CODE('```\n'),
    Token.LINE('\n'),
    Token.LINE('Hello world.\n'),
    Token.AFTER_FILE(),
]


def test_split_rules():
    rules = SplitRules({
        SlideType.SLIDE: [Token.TITLE(level=1)],
        SlideType.FRAGMENT: [Token.TITLE(), Token.SPLIT()],
        SlideType.SUBSLIDE:  [Token.TITLE(level=2)],
    })

    match = rules.match(Token.TITLE('# foo', level=1))
    assert match == (True, SlideType.SLIDE)

    match = rules.match(Token.TITLE('## foo', level=2))
    assert match == (True, SlideType.SUBSLIDE)

    match = rules.match(Token.TITLE('### foo', level=3))
    assert match == (True, SlideType.FRAGMENT)

    match = rules.match(Token.SPLIT())
    assert match == (True, SlideType.FRAGMENT)

    match = rules.match(Token.FILE())
    assert match == (False, None)


def test_clean_source():
    source = [
        '\n',
        '\r\n',
        'foo\n',
        '\n',
        'bar\n',
        '\n',
    ]
    ret = clean_source(source)
    assert ret != source  # no in-place change
    assert ret == ['foo\n', '\n', 'bar']

    source = [
        'foo\n',
        'bar\n',
    ]
    assert clean_source(source) == ['foo\n', 'bar']

    source = ['\n']
    assert clean_source(source) == []


def test_parse_cells_no_splits():
    cells = list(parse_cells(TOKENS, {}))
    assert cells == [
        Cell('markdown', 'slide', [
            '# 111\n',
            '\n',
            '* bullet point\n',
            '* bullet point 2\n',
            '\n',
            '```python\n',
            'def random():\n',
            '    return 4\n',
            '```\n',
            '\n',
            '---\n',
            '\n',
            '```python\n',
            'print(random())\n',
            '```\n',
            '\n',
            '---\n',
            '\n',
            'Text.\n',
            '## 222\n',
            '\n',
            '```python skip\n',
            'import itertools\n',
            '```\n',
            '\n',
            'Hello world.',
        ]),
    ]


def test_parse_cells_code_split():
    split_rules = {
        SlideType.CONTINUE: [Token.START_CODE(), Token.END_CODE()],
        SlideType.SKIP: [Token.START_CODE(skip=True)],
    }
    cells = list(parse_cells(TOKENS, split_rules))
    assert cells == [
        Cell('markdown', 'slide', [
            '# 111\n', '\n', '* bullet point\n', '* bullet point 2',
        ]),
        Cell('code', '-', ['def random():\n', '    return 4']),
        Cell('markdown', '-', ['---']),
        Cell('code', '-', ['print(random())']),
        Cell('markdown', '-', ['---\n', '\n', 'Text.\n', '## 222']),
        Cell('code', 'skip', ['import itertools']),
        Cell('markdown', '-', ['Hello world.']),
    ]


def test_parse_cells_horizontal_split():
    split_rules = {SlideType.SUBSLIDE: [Token.SPLIT()]}
    cells = list(parse_cells(TOKENS, split_rules))
    assert cells == [
        Cell('markdown', 'slide', [
            '# 111\n',
            '\n',
            '* bullet point\n',
            '* bullet point 2\n',
            '\n',
            '```python\n',
            'def random():\n',
            '    return 4\n',
            '```',
        ]),
        Cell('markdown', 'subslide', [
            '```python\n',
            'print(random())\n',
            '```',
        ]),
        Cell('markdown', 'subslide', [
            'Text.\n',
            '## 222\n',
            '\n',
            '```python skip\n',
            'import itertools\n',
            '```\n',
            '\n',
            'Hello world.',
        ]),
    ]


def test_parse_cells_title_splits():
    split_rules = {
        SlideType.SLIDE: [Token.TITLE(level=1)],
        SlideType.SUBSLIDE: [Token.TITLE(level=2)],
    }
    cells = list(parse_cells(TOKENS, split_rules))
    assert cells == [
        Cell('markdown', 'slide', [
            '# 111\n',
            '\n',
            '* bullet point\n',
            '* bullet point 2\n',
            '\n',
            '```python\n',
            'def random():\n',
            '    return 4\n',
            '```\n',
            '\n',
            '---\n',
            '\n',
            '```python\n',
            'print(random())\n',
            '```\n',
            '\n',
            '---\n',
            '\n',
            'Text.',
        ]),
        Cell('markdown', 'subslide', [
            '## 222\n',
            '\n',
            '```python skip\n',
            'import itertools\n',
            '```\n',
            '\n',
            'Hello world.',
        ]),
    ]

    split_rules = {
        SlideType.SLIDE: [Token.TITLE(level=1)],
        SlideType.SUBSLIDE: [Token.AFTER_TITLE(level=1), Token.TITLE(level=2)],
    }
    cells = list(parse_cells(TOKENS, split_rules))
    assert cells == [
        Cell('markdown', 'slide', ['# 111']),
        Cell('markdown', 'subslide', [
            '* bullet point\n',
            '* bullet point 2\n',
            '\n',
            '```python\n',
            'def random():\n',
            '    return 4\n',
            '```\n',
            '\n',
            '---\n',
            '\n',
            '```python\n',
            'print(random())\n',
            '```\n',
            '\n',
            '---\n',
            '\n',
            'Text.',
        ]),
        Cell('markdown', 'subslide', [
            '## 222\n',
            '\n',
            '```python skip\n',
            'import itertools\n',
            '```\n',
            '\n',
            'Hello world.',
        ]),
    ]


def test_parse_cells_combine_split():
    split_rules = {
        SlideType.SLIDE: [Token.TITLE(level=1)],
        SlideType.SUBSLIDE: [Token.AFTER_TITLE(level=1), Token.TITLE(level=2)],
        SlideType.FRAGMENT: [Token.SPLIT()],
        SlideType.CONTINUE: [Token.START_CODE(), Token.END_CODE()],
        SlideType.SKIP: [Token.START_CODE(skip=True)],
    }
    cells = list(parse_cells(TOKENS, split_rules))
    assert cells == [
        Cell('markdown', 'slide', ['# 111']),
        Cell('markdown', 'subslide', ['* bullet point\n', '* bullet point 2']),
        Cell('code', '-', ['def random():\n', '    return 4']),
        Cell('code', 'fragment', ['print(random())']),
        Cell('markdown', 'fragment', ['Text.']),
        Cell('markdown', 'subslide', ['## 222']),
        Cell('code', 'skip', ['import itertools']),
        Cell('markdown', '-', ['Hello world.']),
    ]


def test_parse_cells_slide_hierarchy():
    tokens = [
        Token.TITLE('# 111\n', level=1),
        Token.AFTER_TITLE(level=1),
        Token.SPLIT('---\n'),
        Token.LINE('foo\n'),
        Token.SPLIT('---\n'),
        Token.LINE('bar\n'),
        Token.TITLE('## secret\n', level=2),
        Token.AFTER_TITLE(level=2),
        Token.LINE('baz\n'),
        Token.TITLE('## secret\n', level=2),
        Token.AFTER_TITLE(level=2),
        Token.START_CODE(language='python'),
        Token.LINE('x = 0\n'),
        Token.END_CODE(),
    ]

    split_rules = {
        SlideType.SLIDE: [Token.TITLE(level=1)],
        SlideType.SUBSLIDE: [Token.AFTER_TITLE(level=1)],
        SlideType.FRAGMENT: [Token.SPLIT()],
        SlideType.CONTINUE: [
            Token.TITLE(level=2),
            Token.START_CODE(),
            Token.END_CODE(),
        ],
        SlideType.SKIP: [Token.AFTER_TITLE(level=2)],
    }
    cells = list(parse_cells(tokens, split_rules))
    assert cells == [
        Cell('markdown', 'slide', ['# 111']),
        Cell('markdown', 'subslide', ['foo']),  # Ignore Token.SPLIT
        Cell('markdown', 'fragment', ['bar']),
        Cell('markdown', '-', ['## secret']),
        Cell('markdown', 'skip', ['baz']),
        Cell('markdown', '-', ['## secret']),  # Go back to normal side
        Cell('code', '-', ['x = 0']),  # START_CODE resets slide type
    ]

    split_rules = {
        SlideType.SLIDE: [Token.TITLE(level=1)],
        SlideType.FRAGMENT: [Token.SPLIT()],
        SlideType.CONTINUE: [Token.START_CODE(), Token.END_CODE()],
        SlideType.SKIP: [Token.AFTER_TITLE(level=2)],
    }
    cells = list(parse_cells(tokens, split_rules))
    assert cells == [
        Cell('markdown', 'slide', ['# 111']),
        Cell('markdown', 'fragment', ['foo']),
        Cell('markdown', 'fragment', ['bar\n', '## secret']),
        Cell('markdown', 'skip', ['baz\n', '## secret']),
        Cell('code', '-', ['x = 0']),
    ]



#!/usr/bin/env python

import argparse
import json
import sys

from minimd.tokenizer import Token
from minimd.tokenizer import tokenize_files
from minimd.utils import open_files

from lucina import format_doc
from lucina import parse_cells
from lucina.cell import SlideType


parser = argparse.ArgumentParser()
parser.add_argument(
    'files', metavar='file', nargs='+', help='Files to compute',
)
parser.add_argument('-o', '--output', default=None)
parser.add_argument('--no-autolaunch', dest='autolaunch', action='store_false')

split_rules = {
    SlideType.SLIDE: [Token.TITLE(level=1)],
    SlideType.SUBSLIDE: [Token.TITLE(level=2)],
    SlideType.CONTINUE: [Token.START_CODE(), Token.END_CODE()],
    SlideType.FRAGMENT: [Token.SPLIT()],
    SlideType.SKIP: [Token.START_CODE(skip=True)],
}


def run(args):
    with open_files(args.files, 'r') as files:
        tokens = tokenize_files(files)
        cells = parse_cells(tokens, split_rules)
        doc = format_doc(cells, args.autolaunch)

    if args.output:
        f = open(args.output, 'w')
    else:
        f = sys.stdout

    with f:
        json.dump(doc, f, indent=4)


def main():
    run(parser.parse_args())


if __name__ == '__main__':
    main()



from lucina.cell import Cell
from lucina.format import format_cell
from lucina.format import format_cells
from lucina.format import format_doc


def test_format_cell():
    cell = Cell('markdown', 'fragment', ['foo  \n', '  bar\n'])
    assert format_cell(cell) == {
        'cell_type': 'markdown',
        'metadata': {
            'scrolled': True,
            'slideshow': {
                'slide_type': 'fragment',
            },
        },
        'source': ['foo  \n', '  bar\n'],
    }

    cell = Cell('markdown', 'subslide', [])
    assert format_cell(cell) == {
        'cell_type': 'markdown',
        'metadata': {
            'scrolled': True,
            'slideshow': {
                'slide_type': 'subslide',
            },
        },
        'source': [],
    }

    cell = Cell('code', 'subslide', ['def random():\n', '    return 4\n'])
    assert format_cell(cell) == {
        'cell_type': 'code',
        'metadata': {
            'scrolled': True,
            'slideshow': {
                'slide_type': 'subslide',
            },
        },
        'source': ['def random():\n', '    return 4\n'],
        'outputs': [],
        'execution_count': None,
    }


def test_format_cells():
    c1 = Cell('markdown', 'fragment', ['foo  \n', '  bar\n'])
    c2 = Cell('code', 'subslide', ['def random():\n', '    return 4\n'])

    assert list(format_cells([c1, c2])) == [format_cell(c1), format_cell(c2)]


def test_format_doc():
    cells = [
        Cell('markdown', 'fragment', ['foo  \n', '  bar\n']),
        Cell('code', 'subslide', ['def random():\n', '    return 4\n']),
    ]

    doc = format_doc(cells)
    assert doc['cells'] == list(format_cells(cells))
    assert (doc['nbformat'], doc['nbformat_minor']) == (4, 2)
    assert doc['metadata']['celltoolbar'] == 'Slideshow'
    assert doc['metadata']['language_info']['name'] == 'python'
    assert doc['metadata']['livereveal']['autolaunch'] is True

    doc = format_doc(cells, autolaunch=False)
    assert doc['metadata']['livereveal']['autolaunch'] is False



from typing import Dict
from typing import Iterable
from typing import List
from typing import Tuple

from minimd.tokenizer import Token

from lucina.cell import Cell
from lucina.cell import CellType
from lucina.cell import SlideType


_KEEP_CONTENT_ON_SPLIT = {Token.TITLE}
_SPLIT_CELL_TYPE = {Token.START_CODE: CellType.CODE}


class SplitRules:
    def __init__(self, split_rules: Dict[SlideType, List[Token]]):
        self.rules = {}
        for slide_type, token_patterns in split_rules.items():
            for token_pattern in token_patterns:
                rules = self.rules.setdefault(token_pattern.type, [])
                rules.append((token_pattern.params, slide_type))

        for rules in self.rules.values():
            rules[:] = sorted(rules, key=lambda r: len(r[0]), reverse=True)

    def match(self, token: Token) -> Tuple[bool, SlideType]:
        for rule, slide_type in self.rules.get(token.type, ()):
            if all(token.params.get(k) == v for k, v in rule.items()):
                return True, slide_type
        return False, None


def clean_source(source: List[str]) -> List[str]:
    lines = source[:]

    while lines and not lines[0].rstrip('\r\n'):
        lines.pop(0)
    while lines and not lines[-1].rstrip('\r\n'):
        lines.pop()

    if lines:
        lines[-1] = lines[-1].rstrip('\r\n')

    return lines


def _split_cells(tokens, split_rules):
    split_rules = SplitRules(split_rules)
    cell_type, lines = CellType.MARKDOWN, []
    for token in tokens:
        split, slide_type = split_rules.match(token)

        if split:
            yield cell_type, lines
            yield None, slide_type
            cell_type = _SPLIT_CELL_TYPE.get(token.type, CellType.MARKDOWN)
            lines = []
            if token.type not in _KEEP_CONTENT_ON_SPLIT:
                continue

        if token.content is not None:
            lines.append(token.content)

    yield cell_type, lines


def parse_cells(
        tokens: Iterable[Token],
        split_rules: Dict[SlideType, List[Token]],
) -> Iterable[Cell]:
    slide_type = SlideType.SLIDE
    for cell_type, content in _split_cells(tokens, split_rules):
        if cell_type is None:
            if slide_type is None or content.rank < slide_type.rank:
                slide_type = content
            continue

        source = clean_source(content)
        if source:
            yield Cell(cell_type, slide_type or SlideType.CONTINUE, source)
            slide_type = None



from typing import Iterable

from lucina.cell import Cell
from lucina.cell import CellType


def format_cell(cell: Cell) -> dict:
    doc = {
        'cell_type': cell.cell_type.value,
        'metadata': {
            'scrolled': True,
            'slideshow': {
                'slide_type': cell.slide_type.value,
            },
        },
        'source': cell.source,
    }

    if cell.cell_type == CellType.CODE:
        doc['outputs'] = []
        doc['execution_count'] = None

    return doc


def format_cells(cells: Iterable[Cell]) -> Iterable[dict]:
    for cell in cells:
        yield format_cell(cell)


def format_doc(cells: Iterable[Cell], autolaunch: bool = True) -> dict:
    return {
        'cells': list(format_cells(cells)),
        'metadata': {
            'celltoolbar': 'Slideshow',
            'kernelspec': {
                'display_name': 'Python 3',
                'language': 'python',
                'name': 'python3'
            },
            'language_info': {
                'codemirror_mode': {
                    'name': 'ipython',
                    'version': 3
                },
                'file_extension': '.py',
                'mimetype': 'text/x-python',
                'name': 'python',
                'nbconvert_exporter': 'python',
                'pygments_lexer': 'ipython3',
                'version': '3.6.5'
            },
            "livereveal": {
                "autolaunch": autolaunch,
            }
        },
        'nbformat': 4,
        'nbformat_minor': 2,
    }
